<?php

namespace NF\Discord;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\ServerException;
use GuzzleHttp\Exception\ConnectException;
use GuzzleHttp\Message\Request;

class Api
{
	const BASE_URL = 'https://discord.com/api/';
	const HTTP_GET = 'GET';
	const HTTP_HEAD = 'HEAD';
	const HTTP_POST = 'POST';
	const HTTP_PATCH = 'PATCH';
	const HTTP_PUT = 'PUT';
	const HTTP_DELETE = 'DELETE';

	protected $token;
	protected $guildId;

	public static $CHANNEL_PATCH_PARAMS = [
		'name',
		'type',
		'position',
		'topic',
		'bitrate',
		'user_limit',
	];

	const CHANNEL_TYPE_TEXT = 0;
	const CHANNEL_TYPE_VOICE = 2;
	const CHANNEL_TYPE_NEWS = 5;
	const CHANNEL_TYPE_BOTH = 'both';

	const MENTION_USER = 0x01;
	const MENTION_USER_NICK = 0x02;
	const MENTION_CHANNEL = 0x03;
	const MENTION_ROLE = 0x04;

	public function __construct()
	{
		$provider = $this->getDiscordConfiguration();
		if ($provider)
		{
			$this->token = $provider->options['token'];
			$this->guildId = $provider->options['guild_id'];
		}
	}

	/**
	 * @return \XF\Entity\ConnectedAccountProvider|null
	 * @throws \XF\PrintableException
	 */
	public function assertDiscordConfigured()
	{
		$provider = $this->getDiscordConfiguration();
		if (!$provider)
		{
			throw \XF::phrasedException('nf_discord_discord_not_configured');
		}

		return $provider;
	}

	/**
	 * @return \XF\Entity\ConnectedAccountProvider|null
	 */
	public function getDiscordConfiguration()
	{
		/** @var \XF\Entity\ConnectedAccountProvider $provider */
		$provider = \XF::app()->em()->find('XF:ConnectedAccountProvider', 'nfDiscord');

		if (empty($provider->options['guild_id']) ||
			empty($provider->options['token']) ||
			empty($provider->options['client_id']) ||
			empty($provider->options['client_secret']))
		{
			return null;
		}

		return $provider;
	}

	public function getGuild()
	{
		return $this->get('guilds/:guildId');
	}

	public function getGuildMembers($limit = 1, $offset = 0)
	{
		$data = ['limit' => $limit, 'offset' => $offset];
		return $this->get('guilds/:guildId/members', $data) ?: [];
	}

	public function getGuildMember($userId)
	{
		return $this->get("guilds/:guildId/members/$userId");
	}

	public function addGuildMember(string $userId, string $accessToken)
	{
		return $this->put("guilds/:guildId/members/$userId", [
			'access_token' => $accessToken,
		]);
	}

	/**
	 * Kicks the specified Discord user from the server.
	 *
	 * @param string $discordId
	 *
	 * @return bool|Request|null
	 */
	public function removeGuildMember(string $discordId)
	{
		return $this->delete("guilds/:guildId/members/$discordId");
	}

	public function isMemberOfGuild($guildId)
	{
		$guilds = $this->get('users/@me/guilds') ?: [];

		foreach ($guilds AS $guild)
		{
			if ($guild['id'] == $guildId)
			{
				return true;
			}
		}

		return false;
	}

	public function isWidgetEnabledForGuild($guild)
	{
		if (is_array($guild) && array_key_exists('embed_enabled', $guild))
		{
			return ($guild['embed_enabled']);
		}
		else
		{
			return false;
		}
	}

	protected function getCacheId($id)
	{
		return 'discord_' . md5($this->guildId) . '_' . $id;
	}

	public function getChannels($type = self::CHANNEL_TYPE_BOTH, $cache = false)
	{
		$cacheObj = $cache ? \XF::app()->cache() : null;
		$cacheId = $this->getCacheId('channels_' . $type);
		if ($cacheObj)
		{
			$channels = $cacheObj->fetch($cacheId);
			if ($channels !== false)
			{
				$channels = @\json_decode($channels, true);
				if (\is_array($channels))
				{
					return $channels;
				}
			}
		}

		$channels = $this->get("guilds/:guildId/channels") ?: [];

		if ($type == self::CHANNEL_TYPE_TEXT)
		{
			$textChannels = [];

			foreach ($channels as $channel)
			{
				if ($channel['type'] == self::CHANNEL_TYPE_TEXT || $channel['type'] == self::CHANNEL_TYPE_NEWS)
				{
					$textChannels[$channel['id']] = $channel;
				}
			}

			$channels = $textChannels;
		}
		else if ($type == self::CHANNEL_TYPE_VOICE)
		{
			$voiceChannels = [];

			foreach ($channels as $channel)
			{
				if ($channel['type'] == self::CHANNEL_TYPE_VOICE)
				{
					$voiceChannels[$channel['id']] = $channel;
				}
			}

			$channels = $voiceChannels;
		}

		if ($cacheObj)
		{
			$cacheObj->save($cacheId, \json_encode($channels), 5 * 60);
		}
		return $channels;
	}

	public function getChannelsForOptions($type = self::CHANNEL_TYPE_BOTH, $cache = false)
	{
		$channels = $this->getChannels($type, $cache);
		$output = [];

		if (!empty($channels))
		{
			foreach ($channels AS $id => $info)
			{
				$output[$id] = $info['name'];
			}
		}

		return $output;
	}

	public function createChannel(array $data)
	{
		$this->validateChannelParams($data);
		return $this->post("guilds/:guildId/channels", $data);
	}

	public function patchGuild($input)
	{
		$data = [
			'name' => '',
			'region' => '',
			'verification_level' => '',
			'afk_channel_id' => '',
			'afk_timeout' => '',
			'icon' => '',
			'owner_id' => '',
		];
		$data = array_merge($input, $data);
	}

	public function patchGuildMemberRoles($userId, array $groups)
	{
		$newGroups = [];
		foreach ($groups as $group)
		{
			if (strlen($group) < 18)
			{
				continue;
			}
			$newGroups[] = $group;
		}

		return $this->patch("guilds/:guildId/members/$userId", ['roles' => $newGroups]);
	}

	/**
	 * @param $userId
	 * @param int $deleteMessagesFromDays how many days of messages to delete from this user? Max is 7.
	 */
	public function banGuildMember($userId, $deleteMessagesFromDays = 0)
	{
		return $this->put("guilds/:guildId/bans/$userId");
	}

	public function unbanGuildMember($userId)
	{
		return $this->delete("guilds/:guildId/bans/$userId");
	}

	public function getInvites()
	{
		return $this->get('guilds/:guildId/invites');
	}

	/**
	 * Remove a role from the selected user
	 *
	 * @param $userId
	 * @param $roleId
	 *
	 * @return bool|mixed
	 */
	public function removeGuildMemberRole($userId, $roleId)
	{
		return $this->delete("guilds/:guildId/members/$userId/roles/$roleId");
	}

	/**
	 * Add a role to the selected user
	 *
	 * @param $userId
	 * @param $roleId
	 *
	 * @return bool|mixed
	 */
	public function addGuildMemberRole($userId, $roleId)
	{
		return $this->put("guilds/:guildId/members/$userId/roles/$roleId");
	}

	public function wipeUserRoles($userId)
	{
		return $this->patch("guilds/:guildId/members/$userId", ['roles' => []]);
	}

	public function modifyUserNick($nick, $userId)
	{
		return $this->patch("guilds/:guildId/members/$userId", ['nick' => $nick], [
			'headers' => [
				'X-Audit-Log-Reason' => \XF::phrase('nfd_forced_nickname_via_forum'),
			],
		]);
	}

	/**
	 * Gets the roles ranked below the bot.
	 * REQUIRES: Manage Roles Permission
	 * @return mixed
	 */
	public function getRoles($filtered = true)
	{
		$roles = $this->get('guilds/:guildId/roles') ?: [];
		if (!is_array($roles))
		{
			return [];
		}
		else
		{
			foreach ($roles as &$role)
			{
				$role['admin'] = false;
				if (($role['permissions'] & Helper::ADMINISTRATOR) != 0)
				{
					$role['admin'] = true;
				}
			}
			return $roles;
		}
	}

	public function getRole($id)
	{
		$roles = $this->getRoles();

		$role = [];
		foreach ($roles as $tempRole)
		{
			if ($tempRole['id'] == $id)
			{
				$role = $tempRole;
				break;
			}
		}
		return $role;
	}

	/**
	 * Sends a create role request to Discord.
	 * Returns an empty role object or error.
	 * @return mixed
	 */
	private function createEmptyRole()
	{
		return $this->post('guilds/:guildId/roles');
	}

	/**
	 * Updates a given role, if allowed and formatted properly.
	 *
	 * @param int $roleId The snowflake role id to update
	 * @param array $roleData
	 *
	 * @return mixed
	 */
	public function updateRole($roleId, array $roleData)
	{
		$role = [
			'name' => 'new role',
			'permissions' => '104061953',
			'position' => '1',
			'color' => '0',
			'hoist' => false,
			'mentionable' => false,
		];
		if (array_key_exists('id', $roleData))
		{
			unset($roleData['id']);
		}
		$role = array_replace($role, $roleData);

		return $this->patchRole($roleId, $role);
	}

	public function createRole($roleData)
	{
		$role = $this->createEmptyRole();
		$role = array_merge($role, [
			'name' => $roleData['name'],
			'permissions' => $roleData['permissions'],
			'position' => $roleData['position'],
			'color' => empty($roleData['color']) ? 0 : $roleData['color'],
		]);

		if (array_key_exists('hoist', $roleData))
		{
			$role['hoist'] = true;
		}

		if (array_key_exists('mentionable', $roleData))
		{
			$role['mentionable'] = $roleData['mentionable'];
		}
		return $this->updateRole($role['id'], $role);
	}

	private function patchRole($id, $roleData)
	{
		return $this->patch("guilds/:guildId/roles/$id", $roleData, [
			'headers' => [
				'X-Audit-Log-Reason' => \XF::phrase('nfd_updated_role_via_forum'),
			],
		]);
	}

	public function deleteRole($id)
	{
		return $this->delete("guilds/:guildId/roles/$id");
	}

	public static function getRolePermissionInputFilters()
	{
		return [
			'CREATE_INSTANT_INVITE' => 'bool',
			'KICK_MEMBERS' => 'bool',
			'BAN_MEMBERS' => 'bool',
			'ADMINISTRATOR' => 'bool',
			'MANAGE_CHANNELS' => 'bool',
			'MANAGE_GUILD' => 'bool',
			'ADD_REACTIONS' => 'bool',
			'READ_MESSAGES' => 'bool',
			'SEND_MESSAGES' => 'bool',
			'SEND_TTS_MESSAGES' => 'bool',
			'MANAGE_MESSAGES' => 'bool',
			'EMBED_LINKS' => 'bool',
			'ATTACH_FILES' => 'bool',
			'READ_MESSAGE_HISTORY' => 'bool',
			'MENTION_EVERYONE' => 'bool',
			'USE_EXTERNAL_EMOJIS' => 'bool',
			'CONNECT' => 'bool',
			'SPEAK' => 'bool',
			'MUTE_MEMBERS' => 'bool',
			'DEAFEN_MEMBERS' => 'bool',
			'MOVE_MEMBERS' => 'bool',
			'USE_VAD' => 'bool',
			'CHANGE_NICKNAME' => 'bool',
			'MANAGE_NICKNAMES' => 'bool',
			'MANAGE_ROLES' => 'bool',
			'MANAGE_WEBHOOKS' => 'bool',
			'MANAGE_EMOJIS' => 'bool',
		];
	}

	public function getChannel($channelId)
	{
		$channel = $this->request(['method' => self::HTTP_GET], 'channels/' . $channelId);

		if (array_key_exists('code', $channel) && $channel['code'] == 0)
		{
			return false;
		}
		else
		{
			return $channel;
		}
	}

	public function patchChannel($channelId, array $data)
	{
		if (count($data) == 0 || !$channelId)
		{
			return false;
		}
		else
		{
			self::validateChannelParams($data);
			return $this->request(['method' => self::HTTP_PATCH], 'channels/' . $channelId, $data);
		}
	}

	public function deleteChannel($channelId)
	{
		return $this->request(['method' => self::HTTP_DELETE], 'channels/' . $channelId);
	}

	public function createMessage($channelId, $content = '', $embed = null)
	{
		$options = [
			'content' => $content,
		];

		if (!is_null($embed))
		{
			$options['embed'] = $embed;
		}

		return $this->request(['method' => self::HTTP_POST, 'skipUA' => true], 'channels/' . $channelId . '/messages', $options);
	}

	public function deleteMessage($channelId, $messageId)
	{
		return $this->request(['method' => self::HTTP_DELETE, 'skipUA' => true], 'channels/' . $channelId . '/messages/' . $messageId);
	}

	public function createDM($recipientId)
	{
		return $this->request(['method' => self::HTTP_POST, 'skipUA' => true], 'users/@me/channels', ['recipient_id' => $recipientId]);
	}

	public function alertChannel($channelId, array $data)
	{
		if (!$channelId || sizeof($data) == 0)
		{
			return false;
		}

		if (strpos($data['message'], '{mention}') !== false)
		{
			$data['mention_type'] = self::MENTION_CHANNEL;
			$data['mention_id'] = $channelId;
		}

		$message = $this->formatMessage($data);

		return $this->createMessage($channelId, $message);
	}

	private function formatMessage(array $messageParams)
	{
		$params = [
			'message' => 'Hello, friends!',
		];

		$params = array_merge($params, $messageParams);
		$message = $params['message'];
		if (array_key_exists('mention_type', $params) && array_key_exists('mention_id', $params))
		{
			$message = str_replace('{mention}',
				$this->formatMention($params['mention_type'], $params['mention_id']),
				$message
			);
		}
		return $message;
	}

	/**
	 * Return a string containing a proper mention for the specific type (defaults to user)
	 *
	 * @param int $mentionType see MENTION_* constants
	 * @param int $mentionId the id to mention
	 *
	 * @return string
	 */
	private function formatMention($mentionType = self::MENTION_USER, $mentionId)
	{
		switch ($mentionType)
		{
			default:
			case self::MENTION_USER:
				return "<@$mentionId>";
				break;
			case self::MENTION_USER_NICK:
				return "<@!$mentionId>";
				break;
			case self::MENTION_CHANNEL:
				return "<#$mentionId>";
				break;
			case self::MENTION_ROLE:
				return "<@&$mentionId>";
				break;
		}
	}

	public static function validateChannelParams(array &$params)
	{
		foreach (array_keys($params) as $key)
		{
			if (!in_array($key, self::$CHANNEL_PATCH_PARAMS))
			{
				unset($params[$key]);
			}
		}

		if (array_key_exists('name', $params))
		{
			if (strlen($params['name']) < 2)
			{
				unset($params['name']);
			}
			else if (strlen($params['name'] > 100))
			{
				$params['name'] = substr($params['name'], 0, 98);
			}
		}

		if (array_key_exists('topic', $params))
		{
			if (strlen($params['topic']) > 1000)
			{
				$params['topic'] = substr($params['topic'], 0, 998);
			}
		}

		if (array_key_exists('user_limit', $params))
		{
			if ($params['user_limit'] < 0)
			{
				$params['user_limit'] = 0;
			}
			else if ($params['user_limit'] > 99)
			{
				$params['user_limit'] = 99;
			}
		}

		if (array_key_exists('bitrate', $params))
		{
			if ($params['bitrate'] < 8000)
			{
				$params['bitrate'] = 8000;
			}
			else if ($params['bitrate'] > 96000)
			{
				$params['bitrate'] = 96000;
			}
		}
	}

	/**
	 * Gets the currently authenticated user (bearer)
	 *
	 * @param bool $accessCode
	 *
	 * @return mixed|null
	 */
	public function getCurrentUser($accessCode = false)
	{
		$options = [];
		if ($accessCode)
		{
			$options = [
				'isBearer' => true,
				'accessCode' => $accessCode,
			];
		}
		return $this->get('users/@me', [], $options) ?: null;
	}

	public function searchUsers($needle, $limit = 20)
	{
		return $this->get('users', ['q' => $needle, 'limit' => $limit]);
	}

	public function getVoiceRegions()
	{
		return $this->get('voice/regions');
	}

	protected function isGuzzle6()
	{
		return version_compare(Client::VERSION, '6.0.0', '>=');
	}

	/**
	 * @param array $conditions
	 * @param string $path
	 * @param array $data
	 * @param bool $throwOnClientErrors
	 *
	 * @return Request|bool|null
	 */
	public function request(array $conditions, $path = '', array $data = [], $throwOnClientErrors = false)
	{
		if (substr($path, 0, 1) == '/' or substr($path, 0, 1) == '\\')
		{
			$path = substr($path, 1, strlen($path) - 1);
		}

		$options = $this->prepareRequestConditions($conditions);
		$path = $this->preparePath($path);

		$client = \XF::app()->http()->createClient([
			'base_url' => self::BASE_URL,
			'verify' => false,
			'defaults' => [
				'timeout' => 20,
				'connect_timeout' => 3,
				'exceptions' => false,
			],
		]);

		if ($options['auth'])
		{
			$options['headers']['Authorization'] = ($options['bearer'] ? 'Bearer ' . $options['accessCode'] : 'Bot ' . $this->token);
		}

		if ($options['skipUA'])
		{
			$options['headers']['User-Agent'] = 'NF: Discord (nixfifty.com)';
		}

		$options['headers']['Content-Type'] = 'application/json';

		if ($this->isGuzzle6())
		{
			if (count($data))
			{
				$options = array_merge($options, ['body' => \json_encode($data)]);
			}

			try
			{
				/** @noinspection PhpUndefinedMethodInspection */
				$response = $client->request($options['method'], self::BASE_URL . $path, $options);
			}
			catch (ClientException $e)
			{
				if ($throwOnClientErrors)
				{
					throw $e;
				}
				return false;
			}
			catch (ServerException $e)
			{
				return false;
			}
		}
		else
		{
			/** @var Request $request */
			$request = $client->createRequest($options['method'], $path);

			if (array_key_exists('headers', $options))
			{
				$request->addHeaders($options['headers']);
			}

			if (count($data))
			{
				$raw = array_key_exists('normalPost', $options);
				$request->setBody(\GuzzleHttp\Stream\Stream::factory($raw ? $data : \json_encode($data)));
			}

			try
			{
				/** @noinspection PhpUndefinedMethodInspection */
				$response = $client->send($request);
			}
			catch (ClientException $e)
			{
				if ($throwOnClientErrors)
				{
					throw $e;
				}
				return false;
			}
			catch (ServerException $e)
			{
				return false;
			}
			catch (ConnectException $e)
			{
				return false;
			}
		}

		if ($response->getStatusCode() == 204)
		{
			return true;
		}

		if ($response->getStatusCode() == 304 || $response->getStatusCode() == 400 || $response->getStatusCode() == 401
			|| $response->getStatusCode() == 403)
		{
			return false;
		}

		$data = \json_decode($response->getBody(), true);
		if (JSON_ERROR_NONE !== json_last_error())
		{
			return false;
		}
		return $data;
	}

	public function get($path = '', $data = [], $options = [])
	{
		$options = array_merge([
			'method' => self::HTTP_GET,
			'bypassAuth' => false,
			'isBearer' => false,
			'skipUA' => false,
		], $options);

		return $this->request($options, $path, $data);
	}

	public function head($path = '', $data = [], $options = [])
	{
		$options = array_merge([
			'method' => self::HTTP_HEAD,
			'bypassAuth' => false,
			'isBearer' => false,
			'skipUA' => false,
		], $options);

		return $this->request($options, $path, $data);
	}

	public function post($path = '', $data = [], $options = [])
	{
		$options = array_merge([
			'method' => self::HTTP_POST,
			'bypassAuth' => false,
			'isBearer' => false,
			'skipUA' => false,
		], $options);

		return $this->request($options, $path, $data);
	}

	public function delete($path = '', $data = [], $options = [])
	{
		$options = array_merge([
			'method' => self::HTTP_DELETE,
			'bypassAuth' => false,
			'isBearer' => false,
			'skipUA' => false,
		], $options);

		return $this->request($options, $path, $data);
	}

	public function put($path = '', $data = [], $options = [])
	{
		$options = array_merge([
			'method' => self::HTTP_PUT,
			'bypassAuth' => false,
			'isBearer' => false,
			'skipUA' => false,
		], $options);

		return $this->request($options, $path, $data);
	}

	public function patch($path = '', $data = [], $options = [])
	{
		$options = array_merge([
			'method' => self::HTTP_PATCH,
			'bypassAuth' => false,
			'isBearer' => false,
			'skipUA' => false,
		], $options);

		return $this->request($options, $path, $data);
	}

	public function prepareRequestConditions(array $conditions = [])
	{
		$options = [
			'method' => self::HTTP_GET,
			'auth' => true,
			'bearer' => false,
			'skipUA' => false,
		];
		if (empty($conditions))
		{
			return $options;
		}
		if (array_key_exists('method', $conditions) && !empty($conditions['method']))
		{
			$options['method'] = $this->sanitizeMethod($conditions['method']);
		}

		if (array_key_exists('bypassAuth', $conditions) && $conditions['bypassAuth'])
		{
			$options['auth'] = false;
		}

		if (array_key_exists('isBearer', $conditions) && $conditions['isBearer'])
		{
			$options['bearer'] = true;
		}

		if (array_key_exists('skipUA', $conditions) && $conditions['skipUA'])
		{
			$options['skipUA'] = true;
		}

		if (array_key_exists('accessCode', $conditions))
		{
			$options['accessCode'] = $conditions['accessCode'];
		}

		if (array_key_exists('normalPost', $conditions))
		{
			$options['normalPost'] = true;
		}
		return $options;
	}

	protected function sanitizeMethod($method)
	{
		if (empty($method))
		{
			return self::HTTP_GET;
		}

		$allowed = [
			self::HTTP_GET,
			self::HTTP_DELETE,
			self::HTTP_PATCH,
			self::HTTP_POST,
			self::HTTP_PUT,
		];

		if (!in_array($method, $allowed))
		{
			return self::HTTP_GET;
		}
		else
		{
			return $method;
		}
	}

	public function preparePath($path)
	{
		$path = str_replace(':guildId', $this->guildId, $path);

		return $path;
	}

	public function getAuthorizeLink($state = '', $clientId = null)
	{
		$provider = $this->assertDiscordConfigured();

		$options = [
			'scope' => implode(' ', $this->_getScopes()),
			'redirect_uri' => $this->getRedirectUrl(),
			'response_type' => 'code',
			'client_id' => !is_null($clientId) ? $clientId : $provider->options['client_id'],
		];

		if (is_string($state) && strlen($state) > 0)
		{
			$options['state'] = $state;
		}

		return self::BASE_URL . 'oauth2/authorize?' . urldecode(http_build_query($options));
	}

	public function getBotAuthorizeLink()
	{
		$provider = $this->assertDiscordConfigured();

		$options = [
			'client_id' => $provider->options['client_id'],
			'scope' => 'bot',
			'permissions' => 8, // admin
		];

		return self::BASE_URL . 'oauth2/authorize?' . urldecode(http_build_query($options));
	}

	public function getTokenLink($code)
	{
		$provider = $this->assertDiscordConfigured();

		$options = [
			'client_id' => $provider->options['client_id'],
			'client_secret' => $provider->options['client_secret'],
			'grant_type' => 'authorization_code',
			'code' => $code,
			'redirect_uri' => $this->getRedirectUrl(),
		];

		return self::BASE_URL . 'oauth2/token?' . urldecode(http_build_query($options));
	}

	public function getAccessToken($code)
	{
		$provider = $this->assertDiscordConfigured();

		$data = [
			'client_id' => $provider->options['client_id'],
			'client_secret' => $provider->options['client_secret'],
			'grant_type' => 'authorization_code',
			'code' => $code,
			'redirect_uri' => $this->getRedirectUrl(),
		];

		return $this->post('oauth2/token', $data, ['bypassAuth' => true, 'skipUA', 'normalPost' => true]);
	}

	public function checkState($state)
	{
		$viewingUser = \XF::visitor();

		$check = Helper::getCRC32($viewingUser->user_id . 'code');
		if ($check == $state)
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	protected function getRedirectUrl()
	{
		return \XF::app()->options()->boardUrl . '/connected_account.php';
	}

	protected function _getScopes($bot = false)
	{
		$scopes = ['identify', 'email', 'guilds', 'guilds.join'];

		if ($bot)
		{
			$scopes += ['bot'];
		}

		return $scopes;
	}

	/**
	 * @return bool
	 */
	public function testDiscordApi()
	{
		return $this->head('') !== false;
	}

	/**
	 * @param bool $assertConfigured
	 *
	 * @return Api|null
	 * @throws \XF\PrintableException
	 */
	public static function factory($assertConfigured = true)
	{
		$class = \XF::extendClass(__CLASS__);
		/** @var \NF\Discord\Api $api */
		$api = new $class();
		if ($assertConfigured)
		{
			$api->assertDiscordConfigured();
		}
		else if (!$api->getDiscordConfiguration())
		{
			return null;
		}
		return $api;
	}
}
