<?php

namespace NF\Discord\ConnectedAccount\Service;

use OAuth\Common\Consumer\CredentialsInterface;
use OAuth\Common\Http\Client\ClientInterface;
use OAuth\Common\Http\Exception\TokenResponseException;
use OAuth\Common\Http\Uri\Uri;
use OAuth\Common\Http\Uri\UriInterface;
use OAuth\Common\Storage\TokenStorageInterface;
use OAuth\OAuth2\Service\AbstractService;
use OAuth\OAuth2\Token\StdOAuth2Token;

class Discord extends AbstractService
{
	const SCOPE_IDENTIFY = 'identify';
	const SCOPE_EMAIL = 'email';
	const SCOPE_GUILDS = 'guilds';
	const SCOPE_JOIN_GUILDS = 'guilds.join';

	const API_URL = 'https://discord.com/api/v6/';

	public function __construct(CredentialsInterface $credentials, ClientInterface $httpClient, TokenStorageInterface $storage, array $scopes = [], UriInterface $baseApiUri = null, $stateParameterInAutUrl = false, $apiVersion = "")
	{
		if ($baseApiUri === null)
		{
			$baseApiUri = new Uri(self::API_URL);
		}

		parent::__construct($credentials, $httpClient, $storage, $scopes, $baseApiUri, $stateParameterInAutUrl, $apiVersion);
	}

	private function getOAuthBaseUri()
	{
		return 'https://discord.com/api/oauth2/';
	}

	public function getAuthorizationEndpoint()
	{
		return new Uri($this->getOAuthBaseUri() . 'authorize');
	}

	public function getAccessTokenEndpoint()
	{
		return new Uri($this->getOAuthBaseUri() . 'token');
	}

	protected function getAuthorizationMethod()
	{
		return static::AUTHORIZATION_METHOD_HEADER_BEARER;
	}

	protected function parseAccessTokenResponse($responseBody)
	{
		$data = json_decode($responseBody, true);

		if ($data === null || !is_array($data))
		{
			throw new TokenResponseException('Unable to parse access token response.');
		}
		else if (array_key_exists('code', $data) && array_key_exists('message', $data))
		{
			throw new TokenResponseException($data['message']);
		}

		$token = new StdOAuth2Token();
		$token->setAccessToken($data['access_token']);
		unset($data['access_token']);

		if (array_key_exists('refresh_token', $data))
		{
			$token->setRefreshToken($data['refresh_token']);
			unset($data['refresh_token']);
		}

		if (array_key_exists('expires_in', $data))
		{
			$token->setEndOfLife(time() + $data['expires_in']);
			unset($data['expires_in']);
		}

		$token->setExtraParams($data);

		return $token;
	}

	public function getAuthorizationUri(array $additionalParameters = [])
	{
		$parameters = array_merge(
			$additionalParameters,
			[
				'client_id' => $this->credentials->getConsumerId(),
				'redirect_uri' => $this->credentials->getCallbackUrl(),
				'response_type' => 'code',
			]
		);

		if ($this->needsStateParameterInAuthUrl())
		{
			if (!isset($parameters['state']))
			{
				$parameters['state'] = $this->generateAuthorizationState();
			}
			$this->storeAuthorizationState($parameters['state']);
		}

		$parameters['scope'] = implode(' ', $this->scopes);
		// Build the url
		$url = clone $this->getAuthorizationEndpoint();
		foreach ($parameters as $key => $val)
		{
			$url->addToQuery($key, $val);
		}
		return $url;
	}
}