<?php

namespace NF\Discord;

use XF\Entity\User;
use XF\Entity\UserConnectedAccount;
use XF\Entity\UserGroup;

class Helper
{
	const CREATE_INSTANT_INVITE = 0x00000001;
	const KICK_MEMBERS = 0x00000002;
	const BAN_MEMBERS = 0x00000004;
	const ADMINISTRATOR = 0x00000008;
	const MANAGE_CHANNELS = 0x00000010;
	const MANAGE_GUILD = 0x00000020;
	const ADD_REACTIONS = 0x00000040;
	const READ_MESSAGES = 0x00000400;
	const SEND_MESSAGES = 0x00000800;
	const SEND_TTS_MESSAGES = 0x00001000;
	const MANAGE_MESSAGES = 0x00002000;
	const EMBED_LINKS = 0x00004000;
	const ATTACH_FILES = 0x00008000;
	const READ_MESSAGE_HISTORY = 0x00010000;
	const MENTION_EVERYONE = 0x00020000;
	const USE_EXTERNAL_EMOJIS = 0x00040000;
	const CONNECT = 0x00100000;
	const SPEAK = 0x00200000;
	const MUTE_MEMBERS = 0x00400000;
	const DEAFEN_MEMBERS = 0x00800000;
	const MOVE_MEMBERS = 0x01000000;
	const USE_VAD = 0x02000000;
	const CHANGE_NICKNAME = 0x04000000;
	const MANAGE_NICKNAMES = 0x08000000;
	const MANAGE_ROLES = 0x10000000;
	const MANAGE_WEBHOOKS = 0x20000000;
	const MANAGE_EMOJIS = 0x40000000;

	public static function getCRC32($input)
	{
		return hash('crc32', $input);
	}

	public function getUserIdsInUserGroup($userGroupId, $limit = 0, $offset = 0)
	{
		return \XF::db()->fetchPairs(\XF::db()->limit('
				SELECT user_id, is_primary
				FROM xf_user_group_relation
				WHERE user_group_id = ?
				ORDER BY user_id
			', $limit, $offset
		), $userGroupId);
	}

	public function setUserServerGroupsById($userId)
	{
		if (\XF::options()->nfDiscordEnableSync)
		{
			/** @var \XF\Entity\User $user */
			$user = \XF::finder('XF:User')->whereId($userId)->fetchOne();

			if (!$user)
			{
				return false;
			}

			/** @var UserConnectedAccount $externalUser */
			$externalUser = $user->ConnectedAccounts['nfDiscord'];
			if (!$externalUser)
			{
				return false;
			}

			$this->setUserRoles($user, $externalUser->provider_key);
		}

		return false;
	}

	public function setUserServerGroupsByConnectedAccount(UserConnectedAccount $userConnectedAccount)
	{
		$options = \XF::options();

		if ($userConnectedAccount->isInsert())
		{
			if ($options->nfDiscordAssocJoinServer
				&& array_key_exists('token', $userConnectedAccount->extra_data)
			)
			{
				$this->addUserToServer($userConnectedAccount->provider_key, $userConnectedAccount->extra_data['token']);
			}

			if ($options->nfDiscordAssocAnnounce)
			{
				$this->announceUserAssociation($userConnectedAccount->User, $userConnectedAccount->provider_key);
			}
		}

		if ($options->nfDiscordEnableSync)
		{
			$this->setUserRoles($userConnectedAccount->User, $userConnectedAccount->provider_key);
		}

		return true;
	}

	public function getMemberRoles($discordId)
	{
		$api = Api::factory(false);
		if (!$api)
		{
			return false;
		}

		$user = $api->getGuildMember($discordId);

		if ($user === false || !array_key_exists('roles', $user))
		{
			return [];
		}

		return $user['roles'];
	}

	public function getUserRoles(User $user, $discordId, $memberRoles = true)
	{
		$userGroups = $user->secondary_group_ids;
		$userGroups = array_merge($userGroups, [$user->user_group_id]);

		$roleIds = [];

		foreach ($userGroups as $userGroup)
		{
			$ids = $this->getDiscordRolesByGroup($userGroup);
			if (is_array($ids))
			{
				$roleIds = array_merge($roleIds, $ids);
			}
			else
			{
				$roleIds[] = $ids;
			}
		}

		if ($memberRoles && !\XF::options()->nfDiscordOverwriteRoles)
		{
			$roleIds = array_merge($this->getMemberRoles($discordId), $roleIds);
		}

		$roleIds = array_unique($roleIds, SORT_NUMERIC);

		return $roleIds;
	}

	protected function setUserRoles(User $user, $discordId)
	{
		$api = Api::factory(false);
		if (!$api)
		{
			return false;
		}

		$roleIds = $this->getUserRoles($user, $discordId);

		if (\XF::options()->nfDiscordUpdateNicks)
		{
			$api->modifyUserNick($user->username, $discordId);
		}

		return $api->patchGuildMemberRoles($discordId, $roleIds);
	}

	protected function getDiscordRolesByGroup($groupId)
	{
		/** @var UserGroup $group */
		$group = \XF::finder('XF:UserGroup')
			->whereId($groupId)
			->fetchOne();

		$roleIds = [];

		if ($group)
		{
			$roleIds += $group->nfd_server_group_ids;
		}

		return $roleIds;
	}

	public function setUsersServerGroupsByIds(array $ids = [])
	{
		if (count($ids) > 0)
		{
			foreach ($ids as $id)
			{
				$this->setUserServerGroupsById($id);
			}
		}
	}

	public function addUserToServer(string $discordId, string $accessToken)
	{
		$api = Api::factory(false);
		if (!$api)
		{
			return false;
		}

		$user = $api->getGuildMember($discordId);
		if ($user === false)
		{
			$api->addGuildMember($discordId, $accessToken);
			return true;
		}

		return false;
	}

	public function kickUserFromServer(string $discordId)
	{
		$api = Api::factory(false);
		if (!$api)
		{
			return false;
		}

		$guildMember = $api->getGuildMember($discordId);
		if ($guildMember === false)
		{
			return false;
		}

		$api->removeGuildMember($discordId);
		return true;
	}

	public function announceUserAssociation(User $user, $discordId)
	{
		$api = Api::factory(false);
		if (!$api)
		{
			return false;
		}

		$phrase = \XF::phrase('nfDiscord_notice.user_association', [
			'discordUser' => "<@{$discordId}>",
			'username' => $user->username
		]);

		$api->createMessage(\XF::options()->nfDiscordAssocAnnounce, $phrase->render('raw'));

		return true;
	}

	/**
	 * Forges the proper permission integer based on permission input
	 *
	 * @param array $permissions
	 *
	 * @return string
	 */
	public function forgePermissions(array $permissions)
	{
		$forged = 0;
		$constants = $this->getPermissionConstants();
		foreach ($permissions as $key => $value)
		{
			if (array_key_exists($key, $constants) && $value)
			{
				$forged |= $constants[$key];
			}
		}
		return $forged;
	}

	public function getPermissionConstants()
	{
		$class = new \ReflectionClass($this);

		return $class->getConstants();
	}

	/**
	 * @noinspection PhpDocMissingThrowsInspection
	 * @return Helper
	 */
	public static function factory()
	{
		$class = \XF::extendClass(__CLASS__);
		return new $class();
	}
}
