<?php

namespace NF\Discord;

use SV\StandardLib\InstallerHelper;
use XF\AddOn\AbstractSetup;
use XF\AddOn\StepRunnerInstallTrait;
use XF\AddOn\StepRunnerUninstallTrait;
use XF\AddOn\StepRunnerUpgradeTrait;
use XF\Db\Schema\Alter;

class Setup extends AbstractSetup
{
	use InstallerHelper;
	use StepRunnerInstallTrait;
	use StepRunnerUpgradeTrait;
	use StepRunnerUninstallTrait;

	public function installStep1()
	{
		$this->applySchemaUpdates();
	}

	public function installStep2()
	{
		$this->applyConnectedAccountProvider();
	}

	public function installStep3()
	{
		$this->applyGlobalPermission('discord', 'linkAccount', 'general', 'editProfile');
	}

	public function postInstall(array &$stateChanges)
	{
		$this->app->jobManager()->enqueueUnique(
			'permissionRebuild',
			'XF:PermissionRebuild',
			[],
			false
		);
	}

	public function applySchemaUpdates()
	{
		$sm = $this->schemaManager();

		foreach ($this->getAlterTables() as $tableName => $callback)
		{
			if ($sm->tableExists($tableName))
			{
				$sm->alterTable($tableName, $callback);
			}
		}
	}

	public function applyConnectedAccountProvider()
	{
		$this->db()->update('xf_user_connected_account', [
			'provider' => 'nfDiscord',
		], 'provider = ?', 'discord', 'ignore');

		$this->db()->insert('xf_connected_account_provider', [
			'provider_id' => 'nfDiscord',
			'provider_class' => 'NF\Discord:Provider\Discord',
			'display_order' => 100,
			'options' => '{}',
		], false, "provider_class = values(provider_class)");

		// migrate XF1 configuration into connected account provider
		$optionMap = [
			'nfDiscordGuildId' => 'guild_id',
			'nfDiscordAppKey' => 'client_id',
			'nfDiscordAppSecret' => 'client_secret',
			'nfDiscordToken' => 'token',
		];
		$xf1Options = \XF::finder('XF:Option')->where('option_id', '=', \array_keys($optionMap))->fetch();
		if ($xf1Options->count() === count($optionMap))
		{
			$options = [];
			foreach ($optionMap as $old => $new)
			{
				/** @var \XF\Entity\Option $xf1Option */
				$xf1Option = $xf1Options[$old];
				$options[$new] = $xf1Option->option_value;
			}

			$this->db()->update('xf_connected_account_provider', [
				'options' => \json_encode($options),
			], 'provider_id = ?', 'nfDiscord');
		}
	}

	public function upgrade2020100Step1()
	{
		$this->applyConnectedAccountProvider();
	}

	public function upgrade2050470Step1()
	{
		$this->applySchemaUpdates();
	}

	public function upgrade2050670Step1()
	{
		$sm = $this->schemaManager();
		$sm->alterTable('xf_user_option', function (Alter $table)
		{
			$table->dropColumns('nf_discord_mirror_alerts');
		});
	}

	public function uninstallStep1()
	{
		$sm = $this->schemaManager();

		foreach ($this->getRemoveAlterTables() as $tableName => $callback)
		{
			if ($sm->tableExists($tableName))
			{
				$sm->alterTable($tableName, $callback);
			}
		}
	}

	public static $supportedAddOns = [
		'XFRM' => true,
		'SV/ReportCentreEssentials' => true,
	];

	/**
	 * @return array
	 */
	protected function getAlterTables()
	{
		$tables = [];

		$tables['xf_forum_watch'] = function (Alter $table)
		{
			$this->addOrChangeColumn($table, 'send_discord', 'tinyint', 3)->setDefault(0);
		};

		$tables['xf_user_group'] = function (Alter $table)
		{
			$this->addOrChangeColumn($table, 'nfd_server_group_ids', 'varchar', 255)->setDefault('');
		};

		$tables['xf_forum'] = function (Alter $table)
		{
			$table->dropColumns(['nf_discord_enable']);
			$this->addOrChangeColumn($table, 'nf_discord_channel_id', 'varchar', 64)->setDefault(0);
			$this->addOrChangeColumn($table, 'nf_discord_include_replies', 'tinyint', 3)->setDefault(0);
		};

		$tables['xf_user_option'] = function (Alter $table)
		{
			$this->addOrChangeColumn($table, 'nf_discord_optout', 'text')->comment('Comma-separated list of alerts from which the user has opted out for Discord notifications. Example: \'post_like,user_trophy\'')->nullable()->setDefault(null);
		};

		$tables['xf_rm_category'] = function (Alter $table)
		{
			$this->addOrChangeColumn($table, 'nf_discord_channel_id', 'varchar', 64)->setDefault(0);
		};

		$tables['xf_sv_report_queue'] = function (Alter $table)
		{
			$this->addOrChangeColumn($table, 'nf_discord_channel_id', 'varchar', 64)->setDefault(0);
			$this->addOrChangeColumn($table, 'nf_discord_new_reports', 'tinyint', 1)->setDefault(1);
			$this->addOrChangeColumn($table, 'nf_discord_reopened_reports', 'tinyint', 1)->setDefault(1);

			$this->addOrChangeColumn($table, 'nf_discord_old_report_first_threshold', 'int')->setDefault(0);
			$this->addOrChangeColumn($table, 'nf_discord_old_report_repeat_threshold', 'int')->setDefault(0);
			$this->addOrChangeColumn($table, 'nf_discord_old_report_timestamp', 'int')->setDefault(0);
			$this->addOrChangeColumn($table, 'nf_discord_old_report_repeating', 'tinyint', 1)->setDefault(0);

			$this->addOrChangeColumn($table, 'nf_discord_open_count', 'int')->setDefault(0);
			$this->addOrChangeColumn($table, 'nf_discord_open_threshold', 'int')->setDefault(0);
			$this->addOrChangeColumn($table, 'nf_discord_open_timestamp', 'int')->setDefault(0);
		};

		return $tables;
	}

	/**
	 * @return array
	 */
	protected function getRemoveAlterTables()
	{
		$tables = [];

		$tables['xf_forum_watch'] = function (Alter $table)
		{
			$table->dropColumns(['send_discord']);
		};

		$tables['xf_user_group'] = function (Alter $table)
		{
			$table->dropColumns(['nfd_server_group_ids']);
		};

		$tables['xf_forum'] = function (Alter $table)
		{
			$table->dropColumns(['nf_discord_channel_id', 'nf_discord_include_replies']);
		};

		$tables['xf_user_option'] = function (Alter $table)
		{
			$table->dropColumns(['nf_discord_optout']);
		};

		$tables['xf_rm_category'] = function (Alter $table)
		{
			$table->dropColumns(['nf_discord_channel_id']);
		};

		$tables['xf_sv_report_queue'] = function (Alter $table)
		{
			$table->dropColumns([
				'nf_discord_channel_id',
				'nf_discord_new_reports',
				'nf_discord_reopened_reports',
				'nf_discord_old_report_first_threshold',
				'nf_discord_old_report_repeat_threshold',
				'nf_discord_old_report_timestamp',
				'nf_discord_old_report_repeating',
				'nf_discord_open_count',
				'nf_discord_open_threshold',
				'nf_discord_open_timestamp',
			]);
		};

		return $tables;
	}
}
